package javafold;

import javax.swing.JOptionPane;
import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.buffer.FoldHandler;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.msg.BufferUpdate;
import org.gjt.sp.jedit.textarea.DisplayManager;
import org.gjt.sp.jedit.textarea.JEditTextArea;

public class JavaFoldPlugin extends EBPlugin {

	public void handleMessage(EBMessage msg) {
		if (msg instanceof BufferUpdate) {
			Buffer buffer = ((BufferUpdate) msg).getBuffer();
			Object what = ((BufferUpdate) msg).getWhat();

			if (what.equals(BufferUpdate.CLOSED) ||
					(what.equals(BufferUpdate.PROPERTIES_CHANGED) &&
						!(buffer.getFoldHandler() instanceof JavaFold))) {

				JavaFold.getBuffers().remove(buffer);
			}
		}
	}

	public void stop() {
		super.stop();

		JavaFold.getBuffers().clear();
	}

	public static void changeCommentsVisibility(Buffer buffer, JEditTextArea textArea, boolean collapse) {
		FoldHandler foldHandler = buffer.getFoldHandler();

		if ( !(foldHandler instanceof JavaFold) ) {
			JOptionPane.showMessageDialog(textArea.getPainter(), "Java Fold is not set for current buffer.", "Warning", JOptionPane.WARNING_MESSAGE);

			return;
		}

		try {
			buffer.readLock();

			DisplayManager displayManager = textArea.getDisplayManager();

			int lines = buffer.getLineCount();

			for (int i = 0; i < lines; i++) {
				if (!buffer.isFoldStart(i)) {
					continue;
				}

				if (((JavaFold) foldHandler).isCommentFold(buffer, i)) {
					// buffer.isFoldStart(i) returned true, so there must be
					// at least 1 more line in the buffer
					// using next line assures that parent fold won't be
					// collapsed if the fold is already collapsed (isLineVisible()
					// might be also used) and more important it's the best way to
					// find the right comment fold in situations like below:
					// { /*
					//    */
					// }
					int nextLine = i + 1;

					if (collapse) {
						displayManager.collapseFold(nextLine);
					} else {
						displayManager.expandFold(nextLine, true);
					}

					// this may not speed things up because getFoldAtLine() method
					// may do the same iteration that i want to skip, but it felt
					// right to do this
					int[] j = buffer.getFoldAtLine(nextLine);

					i += j[1] - j[0];
				}
			}
		} finally {
			buffer.readUnlock();
		}
	}
}
